@@ -20,22 +20,32 @@ import pydoc
 import shutil
 import tempfile
 import unittest
+import datetime
 
-from pyspark import SparkContext
+from pyspark import SparkContext, SQLContext
 from pyspark.sql import SparkSession, Column, Row
-from pyspark.sql.functions import udf
+from pyspark.sql.functions import udf, assert_true, lit, rand
 from pyspark.sql.udf import UserDefinedFunction
-from pyspark.sql.types import StringType, IntegerType, BooleanType, DoubleType, LongType, \
-    ArrayType, StructType, StructField
+from pyspark.sql.types import (
+    StringType,
+    IntegerType,
+    BooleanType,
+    DoubleType,
+    LongType,
+    ArrayType,
+    StructType,
+    StructField,
+    TimestampNTZType,
+    DayTimeIntervalType,
+)
 from pyspark.sql.utils import AnalysisException
 from pyspark.testing.sqlutils import ReusedSQLTestCase, test_compiled, test_not_compiled_message
 from pyspark.testing.utils import QuietTest
 
 
 class UDFTests(ReusedSQLTestCase):
-
     def test_udf_with_callable(self):
-        d = [Row(number=i, squared=i**2) for i in range(10)]
+        d = [Row(number=i, squared=i ** 2) for i in range(10)]
         rdd = self.sc.parallelize(d)
         data = self.spark.createDataFrame(rdd)
 
@@ -46,11 +56,11 @@ class UDFTests(ReusedSQLTestCase):
 
         call = PlusFour()
         pudf = UserDefinedFunction(call, LongType())
-        res = data.select(pudf(data['number']).alias('plus_four'))
-        self.assertEqual(res.agg({'plus_four': 'sum'}).collect()[0][0], 85)
+        res = data.select(pudf(data["number"]).alias("plus_four"))
+        self.assertEqual(res.agg({"plus_four": "sum"}).collect()[0][0], 85)
 
     def test_udf_with_partial_function(self):
-        d = [Row(number=i, squared=i**2) for i in range(10)]
+        d = [Row(number=i, squared=i ** 2) for i in range(10)]
         rdd = self.sc.parallelize(d)
         data = self.spark.createDataFrame(rdd)
 
@@ -60,8 +70,8 @@ class UDFTests(ReusedSQLTestCase):
 
         pfunc = functools.partial(some_func, param=4)
         pudf = UserDefinedFunction(pfunc, LongType())
-        res = data.select(pudf(data['number']).alias('plus_four'))
-        self.assertEqual(res.agg({'plus_four': 'sum'}).collect()[0][0], 85)
+        res = data.select(pudf(data["number"]).alias("plus_four"))
+        self.assertEqual(res.agg({"plus_four": "sum"}).collect()[0][0], 85)
 
     def test_udf(self):
         self.spark.catalog.registerFunction("twoArgs", lambda x, y: len(x) + y, IntegerType())
@@ -69,7 +79,7 @@ class UDFTests(ReusedSQLTestCase):
         self.assertEqual(row[0], 5)
 
         # This is to check if a deprecated 'SQLContext.registerFunction' can call its alias.
-        sqlContext = self.spark._wrapped
+        sqlContext = SQLContext.getOrCreate(self.spark.sparkContext)
         sqlContext.registerFunction("oneArg", lambda x: len(x), IntegerType())
         [row] = sqlContext.sql("SELECT oneArg('test')").collect()
         self.assertEqual(row[0], 4)
@@ -77,21 +87,24 @@ class UDFTests(ReusedSQLTestCase):
     def test_udf2(self):
         with self.tempView("test"):
             self.spark.catalog.registerFunction("strlen", lambda string: len(string), IntegerType())
-            self.spark.createDataFrame(self.sc.parallelize([Row(a="test")]))\
-                .createOrReplaceTempView("test")
+            self.spark.createDataFrame(
+                self.sc.parallelize([Row(a="test")])
+            ).createOrReplaceTempView("test")
             [res] = self.spark.sql("SELECT strlen(a) FROM test WHERE strlen(a) > 1").collect()
             self.assertEqual(4, res[0])
 
     def test_udf3(self):
         two_args = self.spark.catalog.registerFunction(
-            "twoArgs", UserDefinedFunction(lambda x, y: len(x) + y))
+            "twoArgs", UserDefinedFunction(lambda x, y: len(x) + y)
+        )
         self.assertEqual(two_args.deterministic, True)
         [row] = self.spark.sql("SELECT twoArgs('test', 1)").collect()
-        self.assertEqual(row[0], u'5')
+        self.assertEqual(row[0], "5")
 
     def test_udf_registration_return_type_none(self):
         two_args = self.spark.catalog.registerFunction(
-            "twoArgs", UserDefinedFunction(lambda x, y: len(x) + y, "integer"), None)
+            "twoArgs", UserDefinedFunction(lambda x, y: len(x) + y, "integer"), None
+        )
         self.assertEqual(two_args.deterministic, True)
         [row] = self.spark.sql("SELECT twoArgs('test', 1)").collect()
         self.assertEqual(row[0], 5)
@@ -100,20 +113,23 @@ class UDFTests(ReusedSQLTestCase):
         with QuietTest(self.sc):
             with self.assertRaisesRegex(TypeError, "Invalid return type"):
                 self.spark.catalog.registerFunction(
-                    "f", UserDefinedFunction(lambda x, y: len(x) + y, StringType()), StringType())
+                    "f", UserDefinedFunction(lambda x, y: len(x) + y, StringType()), StringType()
+                )
 
     def test_nondeterministic_udf(self):
         # Test that nondeterministic UDFs are evaluated only once in chained UDF evaluations
         import random
+
         udf_random_col = udf(lambda: int(100 * random.random()), IntegerType()).asNondeterministic()
         self.assertEqual(udf_random_col.deterministic, False)
-        df = self.spark.createDataFrame([Row(1)]).select(udf_random_col().alias('RAND'))
+        df = self.spark.createDataFrame([Row(1)]).select(udf_random_col().alias("RAND"))
         udf_add_ten = udf(lambda rand: rand + 10, IntegerType())
-        [row] = df.withColumn('RAND_PLUS_TEN', udf_add_ten('RAND')).collect()
+        [row] = df.withColumn("RAND_PLUS_TEN", udf_add_ten("RAND")).collect()
         self.assertEqual(row[0] + 10, row[1])
 
     def test_nondeterministic_udf2(self):
         import random
+
         random_udf = udf(lambda: random.randint(6, 6), IntegerType()).asNondeterministic()
         self.assertEqual(random_udf.deterministic, False)
         random_udf1 = self.spark.catalog.registerFunction("randInt", random_udf)
@@ -145,12 +161,13 @@ class UDFTests(ReusedSQLTestCase):
     def test_nondeterministic_udf_in_aggregate(self):
         from pyspark.sql.functions import sum
         import random
-        udf_random_col = udf(lambda: int(100 * random.random()), 'int').asNondeterministic()
+
+        udf_random_col = udf(lambda: int(100 * random.random()), "int").asNondeterministic()
         df = self.spark.range(10)
 
         with QuietTest(self.sc):
             with self.assertRaisesRegex(AnalysisException, "nondeterministic"):
-                df.groupby('id').agg(sum(udf_random_col())).collect()
+                df.groupby("id").agg(sum(udf_random_col())).collect()
             with self.assertRaisesRegex(AnalysisException, "nondeterministic"):
                 df.agg(sum(udf_random_col())).collect()
 
@@ -167,7 +184,7 @@ class UDFTests(ReusedSQLTestCase):
         # regression test for SPARK-20685
         self.spark.catalog.registerFunction("add", lambda x, y: x + y, IntegerType())
         row = self.spark.sql("SELECT add(1, 1)").first()
-        self.assertEqual(tuple(row), (2, ))
+        self.assertEqual(tuple(row), (2,))
 
     def test_multiple_udfs(self):
         self.spark.catalog.registerFunction("double", lambda x: x * 2, IntegerType())
@@ -182,9 +199,9 @@ class UDFTests(ReusedSQLTestCase):
     def test_udf_in_filter_on_top_of_outer_join(self):
         left = self.spark.createDataFrame([Row(a=1)])
         right = self.spark.createDataFrame([Row(a=1)])
-        df = left.join(right, on='a', how='left_outer')
-        df = df.withColumn('b', udf(lambda x: 'x')(df.a))
-        self.assertEqual(df.filter('b = "x"').collect(), [Row(a=1, b='x')])
+        df = left.join(right, on="a", how="left_outer")
+        df = df.withColumn("b", udf(lambda x: "x")(df.a))
+        self.assertEqual(df.filter('b = "x"').collect(), [Row(a=1, b="x")])
 
     def test_udf_in_filter_on_top_of_join(self):
         # regression test for SPARK-18589
@@ -203,7 +220,7 @@ class UDFTests(ReusedSQLTestCase):
         # Cross join.
         df = left.join(right, f("a", "b"))
         with self.sql_conf({"spark.sql.crossJoin.enabled": False}):
-            with self.assertRaisesRegex(AnalysisException, 'Detected implicit cartesian product'):
+            with self.assertRaisesRegex(AnalysisException, "Detected implicit cartesian product"):
                 df.collect()
         with self.sql_conf({"spark.sql.crossJoin.enabled": True}):
             self.assertEqual(df.collect(), [Row(a=1, b=1)])
@@ -211,6 +228,7 @@ class UDFTests(ReusedSQLTestCase):
     def test_udf_in_left_outer_join_condition(self):
         # regression test for SPARK-26147
         from pyspark.sql.functions import col
+
         left = self.spark.createDataFrame([Row(a=1)])
         right = self.spark.createDataFrame([Row(b=1)])
         f = udf(lambda a: str(a), StringType())
@@ -239,14 +257,17 @@ class UDFTests(ReusedSQLTestCase):
 
         def runWithJoinType(join_type, type_string):
             with self.assertRaisesRegex(
-                    AnalysisException,
-                    'Using PythonUDF.*%s is not supported.' % type_string):
+                AnalysisException,
+                """Using PythonUDF in join condition of join type %s is not supported"""
+                % type_string,
+            ):
                 left.join(right, [f("a", "b"), left.a1 == right.b1], join_type).collect()
-        runWithJoinType("full", "FullOuter")
-        runWithJoinType("left", "LeftOuter")
-        runWithJoinType("right", "RightOuter")
-        runWithJoinType("leftanti", "LeftAnti")
-        runWithJoinType("leftsemi", "LeftSemi")
+
+        runWithJoinType("full", "FULL OUTER")
+        runWithJoinType("left", "LEFT OUTER")
+        runWithJoinType("right", "RIGHT OUTER")
+        runWithJoinType("leftanti", "LEFT ANTI")
+        runWithJoinType("leftsemi", "LEFT SEMI")
 
     def test_udf_as_join_condition(self):
         left = self.spark.createDataFrame([Row(a=1, a1=1, a2=1), Row(a=2, a1=2, a2=2)])
@@ -267,7 +288,8 @@ class UDFTests(ReusedSQLTestCase):
             rdd = self.sc.parallelize(d)
             self.spark.createDataFrame(rdd).createOrReplaceTempView("test")
             self.spark.catalog.registerFunction(
-                "copylist", lambda l: list(l), ArrayType(IntegerType()))
+                "copylist", lambda l: list(l), ArrayType(IntegerType())
+            )
             self.spark.catalog.registerFunction("maplen", lambda d: len(d), IntegerType())
             [(l1, l2)] = self.spark.sql("select copylist(l), maplen(d) from test").collect()
             self.assertEqual(list(range(3)), l1)
@@ -276,7 +298,7 @@ class UDFTests(ReusedSQLTestCase):
     def test_broadcast_in_udf(self):
         bar = {"a": "aa", "b": "bb", "c": "abc"}
         foo = self.sc.broadcast(bar)
-        self.spark.catalog.registerFunction("MYUDF", lambda x: foo.value[x] if x else '')
+        self.spark.catalog.registerFunction("MYUDF", lambda x: foo.value[x] if x else "")
         [res] = self.spark.sql("SELECT MYUDF('c')").collect()
         self.assertEqual("abc", res[0])
         [res] = self.spark.sql("SELECT MYUDF('')").collect()
@@ -288,7 +310,7 @@ class UDFTests(ReusedSQLTestCase):
 
         my_filter = udf(lambda a: a < 2, BooleanType())
         sel = df.select(col("key"), col("value")).filter((my_filter(col("key"))) & (df.value < "2"))
-        self.assertEqual(sel.collect(), [Row(key=1, value='1')])
+        self.assertEqual(sel.collect(), [Row(key=1, value="1")])
 
     def test_udf_with_aggregate_function(self):
         df = self.spark.createDataFrame([(1, "1"), (2, "2"), (1, "2"), (1, "2")], ["key", "value"])
@@ -301,13 +323,16 @@ class UDFTests(ReusedSQLTestCase):
         my_copy = udf(lambda x: x, IntegerType())
         my_add = udf(lambda a, b: int(a + b), IntegerType())
         my_strlen = udf(lambda x: len(x), IntegerType())
-        sel = df.groupBy(my_copy(col("key")).alias("k"))\
-            .agg(sum(my_strlen(col("value"))).alias("s"))\
+        sel = (
+            df.groupBy(my_copy(col("key")).alias("k"))
+            .agg(sum(my_strlen(col("value"))).alias("s"))
             .select(my_add(col("k"), col("s")).alias("t"))
+        )
         self.assertEqual(sel.collect(), [Row(t=4), Row(t=3)])
 
     def test_udf_in_generate(self):
         from pyspark.sql.functions import explode
+
         df = self.spark.range(5)
         f = udf(lambda x: list(range(x)), ArrayType(LongType()))
         row = df.select(explode(f(*df))).groupBy().sum().first()
@@ -345,32 +370,35 @@ class UDFTests(ReusedSQLTestCase):
 
         self.assertListEqual(
             df.selectExpr("add_three(id) AS plus_three").collect(),
-            df.select(add_three("id").alias("plus_three")).collect()
+            df.select(add_three("id").alias("plus_three")).collect(),
         )
 
         # This is to check if a 'SQLContext.udf' can call its alias.
-        sqlContext = self.spark._wrapped
+        sqlContext = SQLContext.getOrCreate(self.spark.sparkContext)
         add_four = sqlContext.udf.register("add_four", lambda x: x + 4, IntegerType())
 
         self.assertListEqual(
             df.selectExpr("add_four(id) AS plus_four").collect(),
-            df.select(add_four("id").alias("plus_four")).collect()
+            df.select(add_four("id").alias("plus_four")).collect(),
         )
 
     @unittest.skipIf(not test_compiled, test_not_compiled_message)  # type: ignore
     def test_register_java_function(self):
         self.spark.udf.registerJavaFunction(
-            "javaStringLength", "test.org.apache.spark.sql.JavaStringLength", IntegerType())
+            "javaStringLength", "test.org.apache.spark.sql.JavaStringLength", IntegerType()
+        )
         [value] = self.spark.sql("SELECT javaStringLength('test')").first()
         self.assertEqual(value, 4)
 
         self.spark.udf.registerJavaFunction(
-            "javaStringLength2", "test.org.apache.spark.sql.JavaStringLength")
+            "javaStringLength2", "test.org.apache.spark.sql.JavaStringLength"
+        )
         [value] = self.spark.sql("SELECT javaStringLength2('test')").first()
         self.assertEqual(value, 4)
 
         self.spark.udf.registerJavaFunction(
-            "javaStringLength3", "test.org.apache.spark.sql.JavaStringLength", "integer")
+            "javaStringLength3", "test.org.apache.spark.sql.JavaStringLength", "integer"
+        )
         [value] = self.spark.sql("SELECT javaStringLength3('test')").first()
         self.assertEqual(value, 4)
 
@@ -380,26 +408,37 @@ class UDFTests(ReusedSQLTestCase):
         df = self.spark.createDataFrame([(1, "a"), (2, "b"), (3, "a")], ["id", "name"])
         df.createOrReplaceTempView("df")
         row = self.spark.sql(
-            "SELECT name, javaUDAF(id) as avg from df group by name order by name desc").first()
-        self.assertEqual(row.asDict(), Row(name='b', avg=102.0).asDict())
+            "SELECT name, javaUDAF(id) as avg from df group by name order by name desc"
+        ).first()
+        self.assertEqual(row.asDict(), Row(name="b", avg=102.0).asDict())
 
     def test_non_existed_udf(self):
         spark = self.spark
-        self.assertRaisesRegex(AnalysisException, "Can not load class non_existed_udf",
-                               lambda: spark.udf.registerJavaFunction("udf1", "non_existed_udf"))
+        self.assertRaisesRegex(
+            AnalysisException,
+            "Can not load class non_existed_udf",
+            lambda: spark.udf.registerJavaFunction("udf1", "non_existed_udf"),
+        )
 
         # This is to check if a deprecated 'SQLContext.registerJavaFunction' can call its alias.
-        sqlContext = spark._wrapped
-        self.assertRaisesRegex(AnalysisException, "Can not load class non_existed_udf",
-                               lambda: sqlContext.registerJavaFunction("udf1", "non_existed_udf"))
+        sqlContext = SQLContext.getOrCreate(self.spark.sparkContext)
+        self.assertRaisesRegex(
+            AnalysisException,
+            "Can not load class non_existed_udf",
+            lambda: sqlContext.registerJavaFunction("udf1", "non_existed_udf"),
+        )
 
     def test_non_existed_udaf(self):
         spark = self.spark
-        self.assertRaisesRegex(AnalysisException, "Can not load class non_existed_udaf",
-                               lambda: spark.udf.registerJavaUDAF("udaf1", "non_existed_udaf"))
+        self.assertRaisesRegex(
+            AnalysisException,
+            "Can not load class non_existed_udaf",
+            lambda: spark.udf.registerJavaUDAF("udaf1", "non_existed_udaf"),
+        )
 
     def test_udf_with_input_file_name(self):
         from pyspark.sql.functions import input_file_name
+
         sourceFile = udf(lambda path: path, StringType())
         filePath = "python/test_support/sql/people1.json"
         row = self.spark.read.json(filePath).select(sourceFile(input_file_name())).first()
@@ -413,19 +452,20 @@ class UDFTests(ReusedSQLTestCase):
 
         sameText = udf(filename, StringType())
 
-        rdd = self.sc.textFile('python/test_support/sql/people.json')
-        df = self.spark.read.json(rdd).select(input_file_name().alias('file'))
-        row = df.select(sameText(df['file'])).first()
+        rdd = self.sc.textFile("python/test_support/sql/people.json")
+        df = self.spark.read.json(rdd).select(input_file_name().alias("file"))
+        row = df.select(sameText(df["file"])).first()
         self.assertTrue(row[0].find("people.json") != -1)
 
         rdd2 = self.sc.newAPIHadoopFile(
-            'python/test_support/sql/people.json',
-            'org.apache.hadoop.mapreduce.lib.input.TextInputFormat',
-            'org.apache.hadoop.io.LongWritable',
-            'org.apache.hadoop.io.Text')
+            "python/test_support/sql/people.json",
+            "org.apache.hadoop.mapreduce.lib.input.TextInputFormat",
+            "org.apache.hadoop.io.LongWritable",
+            "org.apache.hadoop.io.Text",
+        )
 
-        df2 = self.spark.read.json(rdd2).select(input_file_name().alias('file'))
-        row2 = df2.select(sameText(df2['file'])).first()
+        df2 = self.spark.read.json(rdd2).select(input_file_name().alias("file"))
+        row2 = df2.select(sameText(df2["file"])).first()
         self.assertTrue(row2[0].find("people.json") != -1)
 
     def test_udf_defers_judf_initialization(self):
@@ -435,27 +475,29 @@ class UDFTests(ReusedSQLTestCase):
         f = UserDefinedFunction(lambda x: x, StringType())
 
         self.assertIsNone(
-            f._judf_placeholder,
-            "judf should not be initialized before the first call."
+            f._judf_placeholder, "judf should not be initialized before the first call."
         )
 
         self.assertIsInstance(f("foo"), Column, "UDF call should return a Column.")
 
         self.assertIsNotNone(
-            f._judf_placeholder,
-            "judf should be initialized after UDF has been called."
+            f._judf_placeholder, "judf should be initialized after UDF has been called."
         )
 
     def test_udf_with_string_return_type(self):
         add_one = UserDefinedFunction(lambda x: x + 1, "integer")
         make_pair = UserDefinedFunction(lambda x: (-x, x), "struct<x:integer,y:integer>")
         make_array = UserDefinedFunction(
-            lambda x: [float(x) for x in range(x, x + 3)], "array<double>")
+            lambda x: [float(x) for x in range(x, x + 3)], "array<double>"
+        )
 
         expected = (2, Row(x=-1, y=1), [1.0, 2.0, 3.0])
-        actual = (self.spark.range(1, 2).toDF("x")
-                  .select(add_one("x"), make_pair("x"), make_array("x"))
-                  .first())
+        actual = (
+            self.spark.range(1, 2)
+            .toDF("x")
+            .select(add_one("x"), make_pair("x"), make_array("x"))
+            .first()
+        )
 
         self.assertTupleEqual(expected, actual)
 
@@ -499,25 +541,24 @@ class UDFTests(ReusedSQLTestCase):
         def as_double(x):
             return float(x)
 
-        df = (
-            self.spark
-                .createDataFrame(
-                    [(1, "Foo", "foobar", 3.0)], ("one", "Foo", "foobar", "float"))
-                .select(
-                    add_one("one"), add_two("one"),
-                    to_upper("Foo"), to_lower("Foo"),
-                    substr("foobar", lit(0), lit(3)),
-                    trunc("float"), as_double("one")))
+        df = self.spark.createDataFrame(
+            [(1, "Foo", "foobar", 3.0)], ("one", "Foo", "foobar", "float")
+        ).select(
+            add_one("one"),
+            add_two("one"),
+            to_upper("Foo"),
+            to_lower("Foo"),
+            substr("foobar", lit(0), lit(3)),
+            trunc("float"),
+            as_double("one"),
+        )
 
         self.assertListEqual(
             [tpe for _, tpe in df.dtypes],
-            ["int", "double", "string", "string", "string", "bigint", "double"]
+            ["int", "double", "string", "string", "string", "bigint", "double"],
         )
 
-        self.assertListEqual(
-            list(df.first()),
-            [2, 3.0, "FOO", "foo", "foo", 3, 1.0]
-        )
+        self.assertListEqual(list(df.first()), [2, 3.0, "FOO", "foo", "foo", 3, 1.0])
 
     def test_udf_wrapper(self):
         def f(x):
@@ -531,8 +572,9 @@ class UDFTests(ReusedSQLTestCase):
         self.assertEqual(f, f_.func)
         self.assertEqual(return_type, f_.returnType)
 
-        class F(object):
+        class F:
             """Identity"""
+
             def __call__(self, x):
                 return x
 
@@ -552,13 +594,46 @@ class UDFTests(ReusedSQLTestCase):
         self.assertEqual(f, f_.func)
         self.assertEqual(return_type, f_.returnType)
 
+    def test_udf_timestamp_ntz(self):
+        # SPARK-36626: Test TimestampNTZ in Python UDF
+        @udf(TimestampNTZType())
+        def noop(x):
+            assert x == datetime.datetime(1970, 1, 1, 0, 0)
+            return x
+
+        with self.sql_conf({"spark.sql.session.timeZone": "Pacific/Honolulu"}):
+            df = self.spark.createDataFrame(
+                [(datetime.datetime(1970, 1, 1, 0, 0),)], schema="dt timestamp_ntz"
+            ).select(noop("dt").alias("dt"))
+
+            df.selectExpr("assert_true('1970-01-01 00:00:00' == CAST(dt AS STRING))").collect()
+            self.assertEqual(df.schema[0].dataType.typeName(), "timestamp_ntz")
+            self.assertEqual(df.first()[0], datetime.datetime(1970, 1, 1, 0, 0))
+
+    def test_udf_daytime_interval(self):
+        # SPARK-37277: Support DayTimeIntervalType in Python UDF
+        @udf(DayTimeIntervalType(DayTimeIntervalType.DAY, DayTimeIntervalType.SECOND))
+        def noop(x):
+            assert x == datetime.timedelta(microseconds=123)
+            return x
+
+        df = self.spark.createDataFrame(
+            [(datetime.timedelta(microseconds=123),)], schema="td interval day to second"
+        ).select(noop("td").alias("td"))
+
+        df.select(
+            assert_true(lit("INTERVAL '0 00:00:00.000123' DAY TO SECOND") == df.td.cast("string"))
+        ).collect()
+        self.assertEqual(df.schema[0].dataType.simpleString(), "interval day to second")
+        self.assertEqual(df.first()[0], datetime.timedelta(microseconds=123))
+
     def test_nonparam_udf_with_aggregate(self):
         import pyspark.sql.functions as f
 
         df = self.spark.createDataFrame([(1, 2), (1, 2)])
         f_udf = f.udf(lambda: "const_str")
         rows = df.distinct().withColumn("a", f_udf()).collect()
-        self.assertEqual(rows, [Row(_1=1, _2=2, a=u'const_str')])
+        self.assertEqual(rows, [Row(_1=1, _2=2, a="const_str")])
 
     # SPARK-24721
     @unittest.skipIf(not test_compiled, test_not_compiled_message)  # type: ignore
@@ -569,29 +644,35 @@ class UDFTests(ReusedSQLTestCase):
         shutil.rmtree(path)
 
         try:
-            self.spark.range(1).write.mode("overwrite").format('csv').save(path)
-            filesource_df = self.spark.read.option('inferSchema', True).csv(path).toDF('i')
-            datasource_df = self.spark.read \
-                .format("org.apache.spark.sql.sources.SimpleScanSource") \
-                .option('from', 0).option('to', 1).load().toDF('i')
-            datasource_v2_df = self.spark.read \
-                .format("org.apache.spark.sql.connector.SimpleDataSourceV2") \
-                .load().toDF('i', 'j')
+            self.spark.range(1).write.mode("overwrite").format("csv").save(path)
+            filesource_df = self.spark.read.option("inferSchema", True).csv(path).toDF("i")
+            datasource_df = (
+                self.spark.read.format("org.apache.spark.sql.sources.SimpleScanSource")
+                .option("from", 0)
+                .option("to", 1)
+                .load()
+                .toDF("i")
+            )
+            datasource_v2_df = (
+                self.spark.read.format("org.apache.spark.sql.connector.SimpleDataSourceV2")
+                .load()
+                .toDF("i", "j")
+            )
 
-            c1 = udf(lambda x: x + 1, 'int')(lit(1))
-            c2 = udf(lambda x: x + 1, 'int')(col('i'))
+            c1 = udf(lambda x: x + 1, "int")(lit(1))
+            c2 = udf(lambda x: x + 1, "int")(col("i"))
 
-            f1 = udf(lambda x: False, 'boolean')(lit(1))
-            f2 = udf(lambda x: False, 'boolean')(col('i'))
+            f1 = udf(lambda x: False, "boolean")(lit(1))
+            f2 = udf(lambda x: False, "boolean")(col("i"))
 
             for df in [filesource_df, datasource_df, datasource_v2_df]:
-                result = df.withColumn('c', c1)
-                expected = df.withColumn('c', lit(2))
+                result = df.withColumn("c", c1)
+                expected = df.withColumn("c", lit(2))
                 self.assertEqual(expected.collect(), result.collect())
 
             for df in [filesource_df, datasource_df, datasource_v2_df]:
-                result = df.withColumn('c', c2)
-                expected = df.withColumn('c', col('i') + 1)
+                result = df.withColumn("c", c2)
+                expected = df.withColumn("c", col("i") + 1)
                 self.assertEqual(expected.collect(), result.collect())
 
             for df in [filesource_df, datasource_df, datasource_v2_df]:
@@ -601,10 +682,29 @@ class UDFTests(ReusedSQLTestCase):
         finally:
             shutil.rmtree(path)
 
+    # SPARK-42134
+    def test_file_dsv2_with_udf_filter(self):
+        from pyspark.sql.functions import lit
+
+        path = tempfile.mkdtemp()
+        shutil.rmtree(path)
+
+        try:
+            with self.sql_conf({"spark.sql.sources.useV1SourceList": ""}):
+                self.spark.range(1).write.mode("overwrite").format("parquet").save(path)
+                df = self.spark.read.parquet(path).toDF("i")
+                f = udf(lambda x: False, "boolean")(lit(1))
+                result = df.filter(f)
+                self.assertEqual(0, result.count())
+
+        finally:
+            shutil.rmtree(path)
+
     # SPARK-25591
     def test_same_accumulator_in_udfs(self):
-        data_schema = StructType([StructField("a", IntegerType(), True),
-                                  StructField("b", IntegerType(), True)])
+        data_schema = StructType(
+            [StructField("a", IntegerType(), True), StructField("b", IntegerType(), True)]
+        )
         data = self.spark.createDataFrame([[1, 2]], schema=data_schema)
 
         test_accum = self.sc.accumulator(0)
@@ -634,7 +734,7 @@ class UDFTests(ReusedSQLTestCase):
             self.assertEqual(result.collect(), [Row(i=0)])
 
     def test_udf_globals_not_overwritten(self):
-        @udf('string')
+        @udf("string")
         def f():
             assert "itertools" not in str(map)
 
@@ -645,9 +745,10 @@ class UDFTests(ReusedSQLTestCase):
         # and replaces it with '/dev/null'.  See SPARK-26175.
         def task(iterator):
             import sys
+
             res = sys.stdin.read()
             # Because the standard input is '/dev/null', it reaches to EOF.
-            assert res == '', "Expect read EOF from stdin."
+            assert res == "", "Expect read EOF from stdin."
             return iterator
 
         self.sc.parallelize(range(1), 1).mapPartitions(task).count()
@@ -666,24 +767,33 @@ class UDFTests(ReusedSQLTestCase):
         self.assertEqual(r.first()[0], "success")
 
     def test_udf_cache(self):
-        func = lambda x: x
+        def func(x):
+            return x
 
         df = self.spark.range(1)
         df.select(udf(func)("id")).cache()
 
-        self.assertEqual(df.select(udf(func)("id"))._jdf.queryExecution()
-                         .withCachedData().getClass().getSimpleName(), 'InMemoryRelation')
+        self.assertEqual(
+            df.select(udf(func)("id"))
+            ._jdf.queryExecution()
+            .withCachedData()
+            .getClass()
+            .getSimpleName(),
+            "InMemoryRelation",
+        )
 
     # SPARK-34545
     def test_udf_input_serialization_valuecompare_disabled(self):
         def f(e):
             return e[0]
 
-        df = self.spark.createDataFrame([((1.0, 1.0), (1, 1))], ['c1', 'c2'])
-        result = df.select("*", udf(f, DoubleType())("c1").alias('c3'),
-                           udf(f, IntegerType())("c2").alias('c4'))
-        self.assertEqual(result.collect(),
-                         [Row(c1=Row(_1=1.0, _2=1.0), c2=Row(_1=1, _2=1), c3=1.0, c4=1)])
+        df = self.spark.createDataFrame([((1.0, 1.0), (1, 1))], ["c1", "c2"])
+        result = df.select(
+            "*", udf(f, DoubleType())("c1").alias("c3"), udf(f, IntegerType())("c2").alias("c4")
+        )
+        self.assertEqual(
+            result.collect(), [Row(c1=Row(_1=1.0, _2=1.0), c2=Row(_1=1, _2=1), c3=1.0, c4=1)]
+        )
 
     # SPARK-33277
     def test_udf_with_column_vector(self):
@@ -701,10 +811,17 @@ class UDFTests(ReusedSQLTestCase):
             for offheap in ["true", "false"]:
                 with self.sql_conf({"spark.sql.columnVector.offheap.enabled": offheap}):
                     self.assertEquals(
-                        self.spark.read.parquet(path).select(fUdf('id')).head(), Row(0))
+                        self.spark.read.parquet(path).select(fUdf("id")).head(), Row(0)
+                    )
         finally:
             shutil.rmtree(path)
 
+    def test_udf_with_rand(self):
+        # SPARK-40121: rand() with Python UDF.
+        self.assertEqual(
+            len(self.spark.range(10).select(udf(lambda x: x, DoubleType())(rand())).collect()), 10
+        )
+
 
 class UDFInitializationTests(unittest.TestCase):
     def tearDown(self):
@@ -719,11 +836,11 @@ class UDFInitializationTests(unittest.TestCase):
 
         self.assertIsNone(
             SparkContext._active_spark_context,
-            "SparkContext shouldn't be initialized when UserDefinedFunction is created."
+            "SparkContext shouldn't be initialized when UserDefinedFunction is created.",
         )
         self.assertIsNone(
             SparkSession._instantiatedSession,
-            "SparkSession shouldn't be initialized when UserDefinedFunction is created."
+            "SparkSession shouldn't be initialized when UserDefinedFunction is created.",
         )
 
 
@@ -732,7 +849,8 @@ if __name__ == "__main__":
 
     try:
         import xmlrunner  # type: ignore
-        testRunner = xmlrunner.XMLTestRunner(output='target/test-reports', verbosity=2)
+
+        testRunner = xmlrunner.XMLTestRunner(output="target/test-reports", verbosity=2)
     except ImportError:
         testRunner = None
     unittest.main(testRunner=testRunner, verbosity=2)
